use axum::http::StatusCode;

#[derive(sqlx::Type)]
#[sqlx(type_name = "permission", rename_all = "lowercase")]
pub(super) enum PermissionRaw {
    Read,
    Write,
    Manage,
}

#[derive(Clone, Copy, Debug, PartialEq, Eq, PartialOrd, Ord, Default)]
pub enum PermissionType {
    #[default]
    NoPermission = 1,
    Read,
    Write,
    Manage,
}

impl From<Option<PermissionRaw>> for PermissionType {
    fn from(value: Option<PermissionRaw>) -> PermissionType {
        use PermissionRaw as PR;
        match value {
            Some(PR::Read) => PermissionType::Read,
            Some(PR::Write) => PermissionType::Write,
            Some(PR::Manage) => PermissionType::Manage,
            None => PermissionType::NoPermission,
        }
    }
}

impl PermissionType {
    pub fn can_read(self) -> bool {
        self >= PermissionType::Read
    }

    pub fn can_read_guard(self) -> Result<(), StatusCode> {
        if !self.can_read() {
            return Err(StatusCode::NOT_FOUND);
        }
        Ok(())
    }

    pub fn can_write_guard(self) -> Result<(), StatusCode> {
        self.can_read_guard()?;
        if self < PermissionType::Write {
            return Err(StatusCode::FORBIDDEN);
        }
        Ok(())
    }

    pub fn can_manage_guard(self) -> Result<(), StatusCode> {
        self.can_read_guard()?;
        if self < PermissionType::Manage {
            return Err(StatusCode::FORBIDDEN);
        }
        Ok(())
    }
}
